import os
import shutil
from charmhelpers.fetch import (
    UnhandledSource,
    add_source,
    apt_install,
    apt_update,
)
from charmhelpers.core.host import (
    mkdir,
    lsb_release,
)
from charmhelpers.payload.archive import (
    extract,
    archive_dest_default,
)


def install_swift_client():
    # XXX ok, this isn't nice. But we need the swiftclient for precise and
    # the cloud archive is a safe place to get it
    if lsb_release()['DISTRIB_CODENAME'] == 'precise':
        add_source('cloud:precise-updates/folsom', '5EDB1B62EC4926EA')
    apt_update()
    apt_install("python-swiftclient")

try:
    import swiftclient
except ImportError:
    install_swift_client()
    import swiftclient


class SwiftFetchHandler():
    """Handler for swift"""
    def __init__(self, aurl, user, password, tenant_name):
        self.authurl = aurl
        self.user = user
        self.key = password
        self.tenant = tenant_name

    def get_connection(self):
        return swiftclient.client.Connection(authurl=self.authurl,
                                             user=self.user,
                                             key=self.key,
                                             tenant_name=self.tenant,
                                             auth_version='2.0')

    def can_handle(self):
        try:
            sc = self.get_connection()
            sc.close()
            return True
        except:
            return False

    def download(self, src_container, src_filename, dest):
        try:
            sc = self.get_connection()
            headers, content = sc.get_object(src_container, src_filename,
                                             resp_chunk_size=1024**2)
            with open(dest, 'w') as dest_file:
                # iterate the content
                for chunk in content:
                    dest_file.write(chunk)
        except Exception as e:
            if os.path.isfile(dest):
                os.unlink(dest)
            raise e

    def install(self,  src_container, src_filename):
        dest_dir = os.path.join(os.environ.get('CHARM_DIR'), 'fetched')
        if not os.path.exists(dest_dir):
            mkdir(dest_dir, perms=0755)
        dld_file = os.path.join(dest_dir, src_container)
        try:
            self.download(src_container, src_filename, dld_file)
        except OSError as e:
            raise UnhandledSource(e.strerror)
        dest_path = archive_dest_default(dld_file)
        if os.path.exists(dest_path):
            shutil.rmtree(dest_path)
        return extract(dld_file)
